@@ -6,7 +6,7 @@
  *	Copyright (c) 2003-2004, PostgreSQL Global Development Group
  *	Author: Jan Wieck, Afilias USA INC.
  *
- *	$Id: dbutils.c,v 1.23.2.4 2006/12/06 09:46:09 xfade Exp $
+ *	$Id: dbutils.c,v 1.23.2.5 2009-07-20 22:30:47 cbbrowne Exp $
  * ----------------------------------------------------------------------
  */
 
@@ -479,6 +479,12 @@ slon_appendquery_int(SlonDString * dsp, char *fmt, va_list ap)
 						fmt++;
 						break;
 
+					case 'L':
+						sprintf(buf, "%lld", va_arg(ap, int));
+						dstring_append(dsp, buf);
+						fmt++;
+						break;
+
 					default:
 						dstring_addchar(dsp, '%');
 						dstring_addchar(dsp, *fmt);
