@@ -6,7 +6,7 @@
  *	Copyright (c) 2003-2004, PostgreSQL Global Development Group
  *	Author: Jan Wieck, Afilias USA INC.
  *
- *	$Id: slonik.c,v 1.67.2.11 2007-06-07 19:36:14 wieck Exp $
+ *	$Id: slonik.c,v 1.67.2.20 2009-07-31 19:16:28 cbbrowne Exp $
  *-------------------------------------------------------------------------
  */
 
@@ -42,9 +42,7 @@ SlonikScript *parser_script = NULL;
 int			parser_errors = 0;
 int			current_try_level;
 
-#ifdef WIN32
 static char myfull_path[MAXPGPATH];
-#endif
 static char share_path[MAXPGPATH];
 
 /*
@@ -81,6 +79,7 @@ main(int argc, const char *argv[])
 {
 	extern int	optind;
 	int			opt;
+	char 		script[MAXPGPATH];
 
 	while ((opt = getopt(argc, (char **)argv, "hv")) != EOF)
 	{
@@ -104,22 +103,36 @@ main(int argc, const char *argv[])
 
 	if (parser_errors)
 		usage();
+
+	/* Check PGSHARE for scripts first */
+	strcpy(share_path, PGSHARE);
+
+/* There's no libpgport on 7.x, but on later versions we have other options */
+#if (PG_VERSION_MAJOR > 7)
+
+	/* If we can't find the scripts, try the PostgreSQL share directory */
 #ifndef WIN32
-        strcpy(share_path, PGSHARE);
+	snprintf(script, sizeof(script), "%s/slony1_funcs.sql", share_path);
+	if (access(script, R_OK) != 0)
 #else
-	/*
-	 * We need to find a share directory like PostgreSQL. 
-	 */
-	if (find_my_exec(argv[0],myfull_path) < 0)
-	{
-		printf("full path was unacquirable. '%s'\n", argv[0]);
-		return -1;
-	}
-	else
-	{
-		get_share_path(myfull_path, share_path);
-	}
+	_snprintf(script, sizeof(script), "%s/slony1_funcs.sql", share_path);
+	if (_access(script, 04) != 0)
+
 #endif
+	{
+		if (find_my_exec(argv[0],myfull_path) >= 0)
+		{
+			get_share_path(myfull_path, share_path);
+		}
+		else
+		{
+			printf("full path was unacquirable. '%s'\n", argv[0]);
+			return -1;
+		}
+	}
+
+#endif
+
 	if (optind < argc)
 	{
 		while (optind < argc)
@@ -330,11 +343,16 @@ script_check_stmts(SlonikScript * script, SlonikStmt * hdr)
 							   hdr->stmt_filename, hdr->stmt_lno);
 					}
 
-					if (!stmt->no_spool)
+					if (stmt->no_spool)
 					{
-						if (script_check_adminfo(hdr, stmt->no_id) < 0)
+						printf("Slonik command STORE NODE does not "
+						       "support the SPOOLNODE argument being set "
+						       "to TRUE.\n  See logshipping.html for "
+						       "documentation on setting up log shipping.\n");
 							errors++;
 					}
+					if (script_check_adminfo(hdr, stmt->no_id) < 0)
+						errors++;
 					if (script_check_adminfo(hdr, stmt->ev_origin) < 0)
 						errors++;
 				}
@@ -1263,16 +1281,17 @@ script_exec_stmts(SlonikScript * script, SlonikStmt * hdr)
 					(SlonikStmt_init_cluster *) hdr;
 
 					if (slonik_init_cluster(stmt) < 0)
-			case STMT_REPAIR_CONFIG:
-					{
-						SlonikStmt_repair_config *stmt =
-						(SlonikStmt_repair_config *) hdr;
+						errors++;
+				}
+				break;
 
-						if (slonik_repair_config(stmt) < 0)
-							errors++;
-					}
-					break;
-					errors++;
+			case STMT_REPAIR_CONFIG:
+				{
+					SlonikStmt_repair_config *stmt =
+					(SlonikStmt_repair_config *) hdr;
+
+					if (slonik_repair_config(stmt) < 0)
+						errors++;
 				}
 				break;
 
@@ -1886,12 +1905,12 @@ load_slony_base(SlonikStmt * stmt, int no_id)
 		use_major = 8;
 		use_minor = 0;
 	}
-	else if ((adminfo->pg_version >= 80100) && adminfo->pg_version < 80400)	/* 8.1, 8.2 and 8.3 */
+	else if ((adminfo->pg_version >= 80100) && adminfo->pg_version < 80500)	/* 8.1, 8.2, 8.3, 8.4 */
 	{
 		use_major = 8;
 		use_minor = 1;
 	}
-	else	/* 8.3+ */
+	else	/* 8.5+ */
 	{
 		use_major = 8;
 		use_minor = 1;
@@ -2622,6 +2641,16 @@ slonik_failed_node(SlonikStmt_failed_node * stmt)
 	for (i = 0; i < num_nodes; i++)
 	{
 		nodeinfo[i].no_id = (int)strtol(PQgetvalue(res1, i, 0), NULL, 10);
+
+		if (get_adminfo((SlonikStmt *) stmt, nodeinfo[i].no_id) == NULL)
+		{
+			printf("WARNING: no admin conninfo for node %d - node will not be considered\n",
+				nodeinfo[i].no_id);
+			nodeinfo[i].adminfo = NULL;
+			nodeinfo[i].has_slon = false;
+			continue;
+		}
+
 		nodeinfo[i].adminfo = get_active_adminfo((SlonikStmt *) stmt,
 												 nodeinfo[i].no_id);
 		if (nodeinfo[i].adminfo == NULL)
@@ -2662,7 +2691,7 @@ slonik_failed_node(SlonikStmt_failed_node * stmt)
 	PQclear(res1);
 
 	/*
-	 * For every set we're interested in lookup the direct subscriber nodes.
+	 * For every set we're interested in lookup the subscriber nodes.
 	 */
 	for (i = 0; i < num_sets; i++)
 	{
@@ -2676,11 +2705,9 @@ slonik_failed_node(SlonikStmt_failed_node * stmt)
 					 "select sub_receiver "
 					 "    from \"_%s\".sl_subscribe "
 					 "    where sub_set = %d "
-					 "    and sub_provider = %d "
 					 "    and sub_active and sub_forward; ",
 					 stmt->hdr.script->clustername,
-					 setinfo[i].set_id,
-					 stmt->no_id);
+					 setinfo[i].set_id);
 
 		res3 = db_exec_select((SlonikStmt *) stmt, adminfo1, &query);
 		if (res3 == NULL)
@@ -2724,6 +2751,8 @@ slonik_failed_node(SlonikStmt_failed_node * stmt)
 	 * Execute the failedNode() procedure, first on the backup node, then on
 	 * all other nodes.
 	 */
+	printf("INFO: calling failedNode(%d,%d) on node %d\n",
+			stmt->no_id, stmt->backup_node, stmt->no_id);
 	slon_mkquery(&query,
 				 "select \"_%s\".failedNode(%d, %d); ",
 				 stmt->hdr.script->clustername,
@@ -2739,6 +2768,11 @@ slonik_failed_node(SlonikStmt_failed_node * stmt)
 		if (nodeinfo[i].no_id == stmt->backup_node)
 			continue;
 
+		if (nodeinfo[i].adminfo == NULL)
+			continue;
+
+		printf("INFO: calling failedNode(%d,%d) on node %d\n",
+				stmt->no_id, stmt->backup_node, nodeinfo[i].no_id);
 		if (db_exec_command((SlonikStmt *) stmt, nodeinfo[i].adminfo, &query) < 0)
 		{
 			free(configbuf);
@@ -2752,6 +2786,9 @@ slonik_failed_node(SlonikStmt_failed_node * stmt)
 	 */
 	for (i = 0; i < num_nodes; i++)
 	{
+		if (nodeinfo[i].adminfo == NULL)
+			continue;
+
 		if (db_commit_xact((SlonikStmt *) stmt, nodeinfo[i].adminfo) < 0)
 		{
 			free(configbuf);
@@ -2764,6 +2801,7 @@ slonik_failed_node(SlonikStmt_failed_node * stmt)
 	 * Wait until all slon replication engines that where running have
 	 * restarted.
 	 */
+	printf("INFO: Waiting for slon engines to restart\n");
 	n = 0;
 	while (n < num_nodes)
 	{
@@ -2818,6 +2856,9 @@ slonik_failed_node(SlonikStmt_failed_node * stmt)
 				 stmt->no_id);
 	for (i = 0; i < num_nodes; i++)
 	{
+		if (nodeinfo[i].adminfo == NULL)
+			continue;
+
 		res1 = db_exec_select((SlonikStmt *) stmt, nodeinfo[i].adminfo, &query);
 		if (res1 != NULL)
 		{
@@ -2839,7 +2880,7 @@ slonik_failed_node(SlonikStmt_failed_node * stmt)
 	}
 
 	/*
-	 * For every set determine the direct subscriber with the highest applied
+	 * For every set determine the subscriber with the highest applied
 	 * sync, preferring the backup node.
 	 */
 	for (i = 0; i < num_sets; i++)
@@ -2886,6 +2927,9 @@ slonik_failed_node(SlonikStmt_failed_node * stmt)
 		{
 			int64		ssy_seqno;
 
+			if (setinfo[i].subscribers[j]->adminfo == NULL)
+				continue;
+
 			res1 = db_exec_select((SlonikStmt *) stmt,
 								  setinfo[i].subscribers[j]->adminfo, &query);
 			if (res1 == NULL)
@@ -2963,8 +3007,13 @@ slonik_failed_node(SlonikStmt_failed_node * stmt)
 			setinfo[i].max_node->num_sets++;
 		}
 
+		printf("INFO: Node with highest sync for set %d is %d\n",
+				setinfo[i].set_id, use_node);
+
 		if (use_node != stmt->backup_node)
 		{
+			printf("INFO: switching node %d to temporarily receive set %d from node %d\n",
+					stmt->backup_node, setinfo[i].set_id, use_node);
 			slon_mkquery(&query,
 						 "select \"_%s\".storeListen(%d,%d,%d); "
 						 "select \"_%s\".subscribeSet_int(%d,%d,%d,true); ",
@@ -2984,7 +3033,7 @@ slonik_failed_node(SlonikStmt_failed_node * stmt)
 		rc = -1;
 
 	/*
-	 * Now execute all FAILED_NODE events on the node that had the highest of
+	 * Now execute all FAILED_NODE2 events on the node that had the highest of
 	 * all events alltogether.
 	 */
 	if (max_node_total != NULL)
@@ -3013,6 +3062,9 @@ slonik_failed_node(SlonikStmt_failed_node * stmt)
 	 */
 	for (i = 0; i < num_nodes; i++)
 	{
+		if (nodeinfo[i].adminfo == NULL)
+			continue;
+
 		if (db_commit_xact((SlonikStmt *) stmt, nodeinfo[i].adminfo) < 0)
 			rc = -1;
 	}
@@ -3921,7 +3973,7 @@ slonik_ddl_script(SlonikStmt_ddl_script * stmt)
 		}
 		strncpy(dest, dstring_data(&script) + startpos, endpos-startpos);
 		dest[STMTS[stmtno]-startpos] = 0;
-		slon_mkquery(&query, dest);
+		slon_mkquery(&query, "%s", dest);
 		printf("DDL Statement %d: (%d,%d) [%s]\n", stmtno, startpos, endpos, dest);
 		free(dest);
 
