@@ -3,10 +3,10 @@
  *
  *	Our own datatype for safe storage of transaction ID's.
  *
- *	Copyright (c) 2003-2006, PostgreSQL Global Development Group
+ *	Copyright (c) 2003-2004, PostgreSQL Global Development Group
  *	Author: Jan Wieck, Afilias USA INC.
  *
- *	$Id: xxid.c,v 1.10.2.1 2006-01-06 17:07:48 cbbrowne Exp $
+ *	$Id: xxid.c,v 1.12.2.4 2009-07-23 18:30:04 wieck Exp $
  *-------------------------------------------------------------------------
  */
 
@@ -18,6 +18,18 @@
 #include "access/xact.h"
 #include "access/transam.h"
 #include "executor/spi.h"
+#ifdef HAVE_GETACTIVESNAPSHOT
+#include "utils/snapmgr.h"
+#endif
+
+#ifdef PG_MODULE_MAGIC
+PG_MODULE_MAGIC;
+#endif
+
+/* -- Change from PostgreSQL Ver 8.3 -- */
+#if !((PG_VERSION_MAJOR > 8) || ((PG_VERSION_MAJOR == 8) && (PG_VERSION_MINOR >= 3)))
+#define SET_VARSIZE(datum, size) (VARATT_SIZEP(datum)=(size))
+#endif
 
 
 #ifndef PG_GETARG_TRANSACTIONID
@@ -30,7 +42,6 @@
 
 typedef struct
 {
-	int32		varsz;
 	TransactionId xmin;
 	TransactionId xmax;
 	int			nxip;
@@ -225,6 +236,10 @@ _Slony_I_getCurrentXid(PG_FUNCTION_ARGS)
 Datum
 _Slony_I_getMinXid(PG_FUNCTION_ARGS)
 {
+#ifdef HAVE_GETACTIVESNAPSHOT
+	Snapshot	SerializableSnapshot = GetActiveSnapshot();
+#endif
+
 	if (SerializableSnapshot == NULL)
 		elog(ERROR, "Slony-I: SerializableSnapshot is NULL in getMinXid()");
 
@@ -238,6 +253,9 @@ _Slony_I_getMinXid(PG_FUNCTION_ARGS)
 Datum
 _Slony_I_getMaxXid(PG_FUNCTION_ARGS)
 {
+#ifdef HAVE_GETACTIVESNAPSHOT
+	Snapshot	SerializableSnapshot = GetActiveSnapshot();
+#endif
 	if (SerializableSnapshot == NULL)
 		elog(ERROR, "Slony-I: SerializableSnapshot is NULL in getMaxXid()");
 
@@ -257,6 +275,7 @@ _Slony_I_xxid_snapshot_in(PG_FUNCTION_ARGS)
 	int			a_used = 0;
 	TransactionId xmin;
 	TransactionId xmax;
+	void		 *result;
 	xxid_snapshot *snap;
 	int			size;
 
@@ -314,15 +333,16 @@ _Slony_I_xxid_snapshot_in(PG_FUNCTION_ARGS)
 	}
 
 	size = offsetof(xxid_snapshot, xip) + sizeof(TransactionId) * a_used;
-	snap = (xxid_snapshot *) palloc(size);
-	snap->varsz = size;
+	result = (void *) palloc(size + VARHDRSZ);
+	SET_VARSIZE(result, size + VARHDRSZ);
+	snap = (xxid_snapshot *) VARDATA(result);
 	snap->xmin = xmin;
 	snap->xmax = xmax;
 	snap->nxip = a_used;
 	if (a_used > 0)
 		memcpy(&(snap->xip[0]), xip, sizeof(TransactionId) * a_used);
 
-	PG_RETURN_POINTER(snap);
+	PG_RETURN_POINTER(result);
 }
 
 
@@ -332,7 +352,7 @@ _Slony_I_xxid_snapshot_in(PG_FUNCTION_ARGS)
 Datum
 _Slony_I_xxid_snapshot_out(PG_FUNCTION_ARGS)
 {
-	xxid_snapshot *snap = (xxid_snapshot *) PG_GETARG_VARLENA_P(0);
+	xxid_snapshot *snap = (xxid_snapshot *) VARDATA(PG_DETOAST_DATUM((PG_GETARG_DATUM(0))));
 
 	char	   *str = palloc(28 + snap->nxip * 13);
 	char	   *cp = str;
@@ -360,7 +380,7 @@ Datum
 _Slony_I_xxid_lt_snapshot(PG_FUNCTION_ARGS)
 {
 	TransactionId value = PG_GETARG_TRANSACTIONID(0);
-	xxid_snapshot *snap = (xxid_snapshot *) PG_GETARG_VARLENA_P(1);
+	xxid_snapshot *snap = (xxid_snapshot *) VARDATA(PG_DETOAST_DATUM((PG_GETARG_DATUM(1))));
 	int			i;
 
 	if (TransactionIdPrecedes(value, snap->xmin))
@@ -386,7 +406,7 @@ Datum
 _Slony_I_xxid_ge_snapshot(PG_FUNCTION_ARGS)
 {
 	TransactionId value = PG_GETARG_TRANSACTIONID(0);
-	xxid_snapshot *snap = (xxid_snapshot *) PG_GETARG_VARLENA_P(1);
+	xxid_snapshot *snap = (xxid_snapshot *) VARDATA(PG_DETOAST_DATUM((PG_GETARG_DATUM(1))));
 	int			i;
 
 	if (TransactionIdEquals(value, snap->xmax))
